<?php

namespace Drupal\academic_marksheet\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\academic_marksheet\Form\SubjectsForm;

/**
 * Class StudentSubjectsController.
 *
 * Provides functionality to assign subjects to students.
 */
class StudentSubjectsController extends ControllerBase {

  /**
   * Database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Constructs a new StudentSubjectsController.
   */
  public function __construct(Connection $database) {
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database')
    );
  }

  /**
   * Returns the page that lists students and allows assigning subjects.
   */
  public function assignSubjects() {
    // Fetch students from student_details table.
    $query = $this->database->select('student_details', 'sd')
      ->fields('sd', ['roll_no', 'student_name'])
      ->execute();

    $students = $query->fetchAll();

    // Pass students to the form.
    return \Drupal::formBuilder()->getForm('\Drupal\academic_marksheet\Form\SubjectsForm', $students);
  }

  /**
   * Lists all students with their assigned subjects.
   *
   * @return array
   *   A render array for the students with assigned subjects.
   */
  public function listAssignedSubjects() {
    // Query to get students with their assigned subjects.
    $query = $this->database->select('student_details', 'sd')
      ->fields('sd', ['roll_no', 'student_name']);
    $query->leftJoin('student_subjects', 'ss', 'sd.roll_no = ss.student_roll_no');
    $query->leftJoin('faculty', 'f', 'ss.subject_id = f.id');
    $query->fields('f', ['subject_name']);
    $query->orderBy('sd.student_name');

    // Add pager to the query (limit results per page).
    // Fetch 10 results per page.
    $query->range(0, 10);

    // Execute the query.
    $result = $query->execute();

    // Process the result to group subjects by students.
    $students = [];
    foreach ($result as $record) {
      $students[$record->roll_no]['student_name'] = $record->student_name;
      $students[$record->roll_no]['subjects'][] = $record->subject_name;
    }

    // Prepare header for the table.
    $header = [
      ['data' => $this->t('Student Roll No'), 'field' => 'roll_no'],
      ['data' => $this->t('Student Name'), 'field' => 'student_name'],
      ['data' => $this->t('Assigned Subjects')],
    ];

    // Prepare rows for the table.
    $rows = [];
    foreach ($students as $roll_no => $data) {
      $rows[] = [
        'data' => [
          'roll_no' => $roll_no,
          'student_name' => $data['student_name'],
          'subjects' => isset($data['subjects']) ? implode(', ', $data['subjects']) : $this->t('No subjects assigned'),
        ],
      ];
    }

    return [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No students or subjects found.'),
      '#tablesort' => TRUE,
      '#pager' => ['#type' => 'pager'],
    ];
  }

}
