<?php

namespace Drupal\academic_marksheet\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\taxonomy\Entity\Term;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\File\Exception\FileException;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides form to add Students data.
 */
class StudentDetailsForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'student_details_form';
  }

  protected $requestStack;

  /**
   * Constructor to inject the request stack service.
   */
  public function __construct(RequestStack $request_stack) {
    $this->requestStack = $request_stack;
  }

  /**
   *
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('request_stack')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Manual student details form.
    $form['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Student Name'),
    ];

    $form['roll_no'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Roll Number'),
    ];

    $form['dob'] = [
      '#type' => 'date',
      '#title' => $this->t('Date of Birth'),
    ];

    $form['address'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Address'),
    ];

    // Course dropdown (taxonomy reference).
    $form['course'] = [
      '#type' => 'select',
      '#title' => $this->t('Course'),
      '#options' => ['' => $this->t('-Select Course-')] + $this->getTaxonomyOptions('courses'),
    ];

    // Semester dropdown (taxonomy reference).
    $form['sem_id'] = [
      '#type' => 'select',
      '#title' => t('Semester'),
      '#options' => ['' => $this->t('-Select Semester-')] + $this->getTaxonomyOptions('semesters'),
    ];

    // Academic Year dropdown (taxonomy reference).
    $form['academic_year'] = [
      '#type' => 'select',
      '#title' => t('Academic Year'),
      '#options' => ['' => $this->t('-Select Academic-')] + $this->getTaxonomyOptions('academic_year'),
    ];

    // Submit button for manual entry.
    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add Student'),
    ];

    // CSV upload field.
    $form['student_csv'] = [
      '#type' => 'file',
      '#title' => $this->t('Upload Students CSV'),
      '#description' => $this->t('Upload a CSV file containing student details (Name, Roll Number, DOB, Address, Course, Semester, Academic Year).'),
    ];

    // Submit button for CSV upload.
    $form['upload_csv'] = [
      '#type' => 'submit',
      '#value' => $this->t('Upload CSV'),
      '#submit' => ['::uploadCsvSubmit'],
    ];

    return $form;
  }

  /**
   * Helper function to get taxonomy options for dropdown.
   */
  protected function getTaxonomyOptions($vocabulary) {
    $terms = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->loadTree($vocabulary);
    $options = [];
    foreach ($terms as $term) {
      $options[$term->tid] = $term->name;
    }
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    $clicked_button = $triggering_element['#value'] ?? '';

    // Manual entry validation.
    if ($clicked_button == 'Add Student') {
      if (empty($form_state->getValue('name'))) {
        $form_state->setErrorByName('name', $this->t('Student Name is required.'));
      }
      if (empty($form_state->getValue('roll_no'))) {
        $form_state->setErrorByName('roll_no', $this->t('Roll Number is required.'));
      }
      if (empty($form_state->getValue('dob'))) {
        $form_state->setErrorByName('dob', $this->t('Date of Birth is required.'));
      }
      if (empty($form_state->getValue('address'))) {
        $form_state->setErrorByName('address', $this->t('Address is required.'));
      }
    }

    // CSV file validation.
    if ($clicked_button === $this->t('Upload CSV')) {
      $request = $this->requestStack->getCurrentRequest();
      $uploaded_file = $request->files->get('files');

      if (isset($uploaded_file['student_csv'])) {
        $file = \Drupal::service('file_system')->realpath($uploaded_file['student_csv']->getPathname());

        if (empty($file)) {
          $form_state->setErrorByName('student_csv', $this->t('Please upload a CSV file.'));
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $name = $form_state->getValue('name');
    $roll_no = $form_state->getValue('roll_no');
    $dob = $form_state->getValue('dob');
    $address = $form_state->getValue('address');
    // Store term ID.
    $course_tid = (int)$form_state->getValue('course');
    // Store term ID.
    $sem_id =(int) $form_state->getValue('sem_id');
    // Store term ID.
    $academic_year_tid = (int)$form_state->getValue('academic_year');

    // Check for existing roll number.
    $query = \Drupal::database()->select('student_details', 'sd')
      ->fields('sd', ['roll_no'])
      ->condition('roll_no', $roll_no)
      ->execute()
      ->fetchField();

    if ($query) {
      \Drupal::messenger()->addError($this->t('Roll Number %roll_no already exists. No duplicate entries allowed.', ['%roll_no' => $roll_no]));
    }
    else {
      // Insert data into the database with term IDs.
      \Drupal::database()->insert('student_details')
        ->fields([
          'student_name' => $name,
          'roll_no' => $roll_no,
          'dob' => $dob,
          'address' => $address,
          'course' => $course_tid,
          'sem_id' => $sem_id,
          'academic_year' => $academic_year_tid,
        ])
        ->execute();

      \Drupal::messenger()->addStatus($this->t('Student details added successfully.'));
    }
  }

  /**
   * }.
   */
  public function uploadCsvSubmit(array &$form, FormStateInterface $form_state) {
    $request = $this->requestStack->getCurrentRequest();
    $uploaded_file = $request->files->get('files');
    $file = \Drupal::service('file_system')->realpath($uploaded_file['student_csv']->getPathname());

    if ($file) {
      try {
        if (($handle = fopen($file, 'r')) !== FALSE) {
          // Skip header row.
          fgetcsv($handle);

          while (($data = fgetcsv($handle, 1000, ",")) !== FALSE) {
            if (count($data) == 7) {
              $course_tid = $this->getTermIdByName('courses', $data[4]);
              $sem_id = $this->getTermIdByName('semesters', $data[5]);
              $academic_year_tid = $this->getTermIdByName('academic_year', $data[6]);

              if ($course_tid && $sem_id && $academic_year_tid) {
                $roll_no_exists = \Drupal::database()->select('student_details', 'sd')
                  ->fields('sd', ['roll_no'])
                  ->condition('roll_no', $data[1])
                  ->execute()
                  ->fetchField();

                if ($roll_no_exists) {
                  \Drupal::messenger()->addWarning($this->t('Roll Number %roll_no already exists. Skipping duplicate entry.', ['%roll_no' => $data[1]]));
                }
                else {
                  \Drupal::database()->insert('student_details')
                    ->fields([
                      'student_name' => $data[0],
                      'roll_no' => $data[1],
                      'dob' => $data[2],
                      'address' => $data[3],
                      'course' => $course_tid,
                      'sem_id' => $sem_id,
                      'academic_year' => $academic_year_tid,
                    ])
                    ->execute();
                }
              }
              else {
                \Drupal::messenger()->addError($this->t('Invalid taxonomy terms for Course, Semester, or Academic Year in CSV row: %row.', ['%row' => implode(', ', $data)]));
              }
            }
            else {
              \Drupal::messenger()->addError($this->t('Invalid CSV format. Expected 7 columns, got %count in row: %row.', ['%count' => count($data), '%row' => implode(', ', $data)]));
            }
          }

          fclose($handle);
          \Drupal::messenger()->addStatus($this->t('Student details from CSV added successfully.'));
        }
      }
      catch (FileException $e) {
        \Drupal::messenger()->addError($this->t('There was a problem uploading the CSV file.'));
      }
    }
  }

  /**
   * Get taxonomy term ID by name and vocabulary.
   * If the term does not exist, create it.
   *
   * @param string $vocabulary
   *   The machine name of the taxonomy vocabulary.
   * @param string $term_name
   *   The name of the term to search for or add.
   *
   * @return int
   *   The term ID.
   */
  private function getTermIdByName($vocabulary, $term_name) {
    $term_name = trim($term_name);

    // Load the term by its name and vocabulary.
    $terms = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->loadByProperties([
      'name' => $term_name,
      'vid' => $vocabulary,
    ]);

    if (!empty($terms)) {
      // Return the first matching term ID.
      $term = reset($terms);
      return $term->id();
    }

    // If no term exists, create a new term.
    $term = Term::create([
      'vid' => $vocabulary,
      'name' => $term_name,
    ]);

    try {
      $term->save();
      \Drupal::messenger()->addStatus($this->t('New term "%term_name" added to the %vocabulary vocabulary.', [
        '%term_name' => $term_name,
        '%vocabulary' => $vocabulary,
      ]));
      return $term->id();
    }
    catch (\Exception $e) {
      \Drupal::messenger()->addError($this->t('Failed to create new term "%term_name" in %vocabulary: %error', [
        '%term_name' => $term_name,
        '%vocabulary' => $vocabulary,
        '%error' => $e->getMessage(),
      ]));
      return FALSE;
    }
  }

}
