<?php

namespace Drupal\academic_marksheet\Form;

use Drupal\Core\Database\Connection;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class SubjectsForm.
 *
 * Provides form to assign subjects to students.
 */
class SubjectsForm extends FormBase {

  /**
   * Database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Constructor.
   */
  public function __construct(Connection $database) {
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'assign_subjects_form';
  }

  /**
   * Fetch students data from the database.
   *
   * @return array
   *   An array of students with roll number and student name.
   */
  protected function getStudents() {
    $query = $this->database->select('student_details', 's')
      ->fields('s', ['roll_no', 'student_name'])
      ->execute();
    return $query->fetchAll();
  }

  /**
   * Build form for assigning subjects.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $students = $this->getStudents();

    $form['instructions'] = [
      '#markup' => $this->t('Select subjects for each student:'),
    ];

    $form['students'] = [
      '#type' => 'container',
      '#tree' => TRUE,
    ];

    // Fetch available subjects from the faculty table.
    $subjects_query = $this->database->select('faculty', 'f')
      ->fields('f', ['id', 'subject_name'])
      ->execute();
    $subjects = $subjects_query->fetchAllKeyed();

    foreach ($students as $student) {
      $form['students'][$student->roll_no] = [
        '#type' => 'fieldset',
        '#title' => $this->t('Assign subjects for @name (@roll)', [
          '@name' => $student->student_name,
          '@roll' => $student->roll_no,
        ]),
      ];

      $form['students'][$student->roll_no]['subjects'] = [
        '#type' => 'checkboxes',
        '#options' => $subjects,
        '#title' => $this->t('Select subjects'),
      ];
    }

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Assign Subjects'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $students = $form_state->getValue('students');

    foreach ($students as $roll_no => $subject_data) {
      $selected_subjects = array_filter($subject_data['subjects']);

      foreach ($selected_subjects as $subject_id) {
        $existing = $this->database->select('student_subjects', 'ss')
          ->fields('ss', ['id'])
          ->condition('student_roll_no', $roll_no)
          ->condition('subject_id', $subject_id)
          ->execute()
          ->fetchField();

        if (!$existing) {
          // Insert selected subjects into the student_subjects table.
          $this->database->insert('student_subjects')
            ->fields([
              'student_roll_no' => $roll_no,
              'subject_id' => $subject_id,
            ])
            ->execute();
        }
      }
    }

    $this->messenger()->addStatus($this->t('Subjects assigned successfully.'));
  }

}
